@@ -15,15 +15,18 @@
 
 import errno
 import ferry
-import os
-import sys
+import grp
 import json
 import logging
 import logging.config
+import os
+import pwd
 import requests
 import re
 import shutil
 import StringIO
+import sys
+import yaml
 from requests.exceptions import ConnectionError
 from subprocess import Popen, PIPE
 from ferry.table.prettytable import *
@@ -41,6 +44,7 @@ class CLI(object):
         self.cmds.add_option("-k", "--key", "Specify key directory")
         self.cmds.add_option("-m", "--mode", "Deployment mode")
         self.cmds.add_option("-u", "--upgrade", "Upgrade Ferry")
+        self.cmds.add_option("-b", "--build", "Manually build Ferry images")
         self.cmds.add_cmd("clean", "Clean zombie Ferry processes")
         self.cmds.add_cmd("server", "Start all the servers")
         self.cmds.add_cmd("deploy", "Deploy a service to the cloud")
@@ -210,14 +214,6 @@ class CLI(object):
             logging.error("could not connect to ferry server")
             return "It appears Ferry servers are not running.\nType sudo ferry server and try again."
 
-    """
-    Read the location of the directory containing the keys
-    used to communicate with the containers. 
-    """
-    def _read_key_dir(self):
-        f = open(ferry.install.DEFAULT_DOCKER_KEY, 'r')
-        return f.read().strip()
-
     """
     Connector a specific client/connector via ssh. 
     """
@@ -244,9 +240,10 @@ class CLI(object):
 
         # Now form the ssh command. This just executes in the same shell. 
         if connector_ip:
+            keydir, _ = self._read_key_dir()
             key_opt = '-o StrictHostKeyChecking=no'
             host_opt = '-o UserKnownHostsFile=/dev/null'
-            ident = '-i %s/id_rsa' % self._read_key_dir()
+            ident = '-i %s/id_rsa' % keydir
             dest = '%s@%s' % (self.default_user, connector_ip)
             cmd = "ssh %s %s %s %s" % (key_opt, host_opt, ident, dest)
             logging.warning(cmd)
@@ -321,30 +318,57 @@ class CLI(object):
 
         return json_text
 
+    """
+    Read the location of the directory containing the keys
+    used to communicate with the containers. 
+    """
+    def _read_key_dir(self):
+        f = open(ferry.install.DEFAULT_DOCKER_KEY, 'r')
+        k = f.read().strip().split("://")
+        return k[1], k[0]
+
+    def _using_tmp_ssh_key(self):
+        keydir, tmp = self._read_key_dir()
+        return tmp == "tmp"
+
     def _check_ssh_key(self):
-        keydir = open(ferry.install.DEFAULT_DOCKER_KEY, 'r').read().strip()
+        keydir, _ = self._read_key_dir()
         if keydir == ferry.install.DEFAULT_KEY_DIR:
+            keydir = os.environ['HOME'] + '/.ssh/tmp-ferry'
+            ferry.install.GLOBAL_KEY_DIR = 'tmp://' + keydir
             try:
-                ferry.install.GLOBAL_KEY_DIR = os.environ['HOME'] + '/.ssh/ferry'
-                os.makedirs(os.environ['HOME'] + '/.ssh/ferry')
-                shutil.copy(ferry.install.DEFAULT_KEY_DIR + '/id_rsa',
-                            ferry.install.GLOBAL_KEY_DIR + '/id_rsa')
-                os.chmod(ferry.install.GLOBAL_KEY_DIR + '/id_rsa', 0400)
+                os.makedirs(keydir)
             except OSError as e:
                 if e.errno != errno.EEXIST:
-                    logging.error("Could not create private key (%s)" % os.strerror(e.errno))
+                    logging.error("Could not create ssh directory %s" % keydir)
+                    exit(1)
+            try:
+                shutil.copy(ferry.install.DEFAULT_KEY_DIR + '/id_rsa', keydir + '/id_rsa')
+                shutil.copy(ferry.install.DEFAULT_KEY_DIR + '/id_rsa.pub', keydir + '/id_rsa.pub')
+                            
+                uid = pwd.getpwnam(os.environ['USER']).pw_uid
+                gid = grp.getgrnam(os.environ['USER']).gr_gid
+                os.chown(keydir + '/id_rsa', uid, gid)
+                os.chmod(keydir + '/id_rsa', 0400)
+                os.chown(keydir + '/id_rsa.pub', uid, gid)
+                os.chmod(keydir + '/id_rsa.pub', 0444)
+            except OSError as e:
+                logging.error("Could not copy ssh keys (%s)" % str(e))
+                exit(1)
+            except IOError as e:
+                if e.errno == errno.EACCES:
+                    logging.error("Could not override keys in %s, please type \'ferry clean\' and try again." % keydir)
+                exit(1)
 
             ferry.install._touch_file(ferry.install.DEFAULT_DOCKER_KEY, 
                                       ferry.install.GLOBAL_KEY_DIR)
-            logging.warning("Copied private key to " + ferry.install.GLOBAL_KEY_DIR)
+            logging.warning("Copied ssh keys to " + ferry.install.GLOBAL_KEY_DIR)
 
     """
     This is the command dispatch table. 
     """
     def dispatch_cmd(self, cmd, args, options):
         if(cmd == 'start'):
-            # Check if the user is using the global key directory.
-            # If so, we need to make a copy of the key. 
             self._check_ssh_key()
 
             arg = args.pop(0)
@@ -358,16 +382,21 @@ class CLI(object):
                 # Check if the user wants to use one of the global plans.
                 global_path = FERRY_HOME + '/data/plans/' + arg
 
-                # Check if the user has passed in a file extension.
-                # If not go ahead and add one. 
+                # If the user has not supplied a file extension, look for the
+                # file with a YAML extension
                 file_path = global_path
                 n, e = os.path.splitext(global_path)
                 if e == '':
-                    file_path += '.json'
+                    file_path += '.yaml'
+                    e = 'yaml'
 
                 if os.path.exists(file_path):
-                    json_string = self._read_file_arg(file_path)
-                    json_arg = json.loads(json_string)
+                    if e == 'json':
+                        json_string = self._read_file_arg(file_path)
+                        json_arg = json.loads(json_string)
+                    elif e == 'yaml':
+                        yaml_file = open(file_path, 'r')
+                        json_arg = yaml.load(yaml_file)
                     json_arg['_file_path'] = file_path
                 json_arg['_file'] = arg
             return self._create_stack(json_arg, args)
@@ -384,23 +413,21 @@ class CLI(object):
             self.installer._stop_docker_daemon()
             return msg
         elif(cmd == 'clean'):
+            self._check_ssh_key()
             self.installer._start_docker_daemon()
             self.installer._clean_web()
             self.installer._stop_docker_daemon(force=True)
+            self.installer._reset_ssh_key()
             return 'cleaned ferry'
         elif(cmd == 'inspect'):
             return self._inspect_stack(args[0])
         elif(cmd == 'logs'):
             return self._copy_logs(args[0], args[1])
         elif(cmd == 'server'):
-            self.installer.start_web()
+            self.installer.start_web(options)
             return 'started ferry'
         elif(cmd == 'ssh'):
-            # Check if the user is using the global key directory.
-            # If so, we need to make a copy of the key. 
             self._check_ssh_key()
-
-            # Go ahead and connect to the stack. 
             stack_id = args[0]
             connector_id = None
             if len(args) > 1:
@@ -408,11 +435,16 @@ class CLI(object):
 
             return self._connect_stack(stack_id, connector_id)
         elif(cmd == 'quit'):
+            self._check_ssh_key()
             self._stop_all()
             self.installer.stop_web()
             self.installer._stop_docker_daemon()
+
+            if self._using_tmp_ssh_key():
+                self.installer._reset_ssh_key()
             return 'stopped ferry'
         elif(cmd == 'deploy'):
+            self._check_ssh_key()
             stack_id = args.pop(0)
             return self._deploy_stack(stack_id, args)
         elif(cmd == 'info'):
